@@ -1,29 +1,54 @@
+from __future__ import annotations
+
+import collections
 import dataclasses
 import os
 from pathlib import Path
-from typing import Any, Callable, Dict, Iterator, MutableMapping, Optional, Set, TypeVar
+from typing import Any, Callable, Iterator, Mapping, MutableMapping, TypeVar
 
-import click
 import platformdirs
 import tomlkit
 
 from pdm import termui
-from pdm.exceptions import NoConfigError
-from pdm.utils import get_pypi_source
+from pdm.exceptions import NoConfigError, PdmUsageError
 
 T = TypeVar("T")
+ui = termui.UI()
+
+REPOSITORY = "repository"
 
 
-def load_config(file_path: Path) -> Dict[str, Any]:
-    """Load a nested TOML document into key-value paires
+@dataclasses.dataclass
+class RepositoryConfig:
+    url: str
+    username: str | None = None
+    password: str | None = None
+
+    def __rich__(self) -> str:
+        lines = [f"[cyan]url[/] = {self.url}"]
+        if self.username:
+            lines.append(f"[cyan]username[/] = {self.username}")
+        if self.password:
+            lines.append("[cyan]password[/] = <hidden>")
+        return "\n".join(lines)
+
+
+DEFAULT_REPOSITORIES = {
+    "pypi": RepositoryConfig("https://upload.pypi.org/legacy/"),
+    "testpypi": RepositoryConfig("https://test.pypi.org/legacy/"),
+}
+
+
+def load_config(file_path: Path) -> dict[str, Any]:
+    """Load a nested TOML document into key-value pairs
 
     E.g. ["python"]["path"] will be loaded as "python.path" key.
     """
 
-    def get_item(sub_data: Dict[str, Any]) -> Dict[str, Any]:
+    def get_item(sub_data: Mapping[str, Any]) -> Mapping[str, Any]:
         result = {}
         for k, v in sub_data.items():
-            if getattr(v, "items", None) is not None:
+            if k != REPOSITORY and isinstance(v, Mapping):
                 result.update(
                     {f"{k}.{sub_k}": sub_v for sub_k, sub_v in get_item(v).items()}
                 )
@@ -44,6 +69,9 @@ def ensure_boolean(val: Any) -> bool:
     return bool(val) and val.lower() not in ("false", "no", "0")
 
 
+DEFAULT_PYPI_INDEX = "https://pypi.org/simple"
+
+
 @dataclasses.dataclass
 class ConfigItem:
     """An item of configuration, with following attributes:
@@ -62,9 +90,9 @@ class ConfigItem:
     description: str
     default: Any = _NOT_SET
     global_only: bool = False
-    env_var: Optional[str] = None
+    env_var: str | None = None
     coerce: Callable = str
-    replace: Optional[str] = None
+    replace: str | None = None
 
     def should_show(self) -> bool:
         return self.default is not self._NOT_SET
@@ -73,8 +101,7 @@ class ConfigItem:
 class Config(MutableMapping[str, str]):
     """A dict-like object for configuration key and values"""
 
-    pypi_url, verify_ssl = get_pypi_source()
-    _config_map: Dict[str, ConfigItem] = {
+    _config_map: dict[str, ConfigItem] = {
         "cache_dir": ConfigItem(
             "The root directory of cached files",
             platformdirs.user_cache_dir("pdm"),
@@ -84,13 +111,7 @@ class Config(MutableMapping[str, str]):
             "Check if there is any newer version available",
             True,
             True,
-            coerce=ensure_boolean,
-        ),
-        "auto_global": ConfigItem(
-            "Use global package implicitly if no local project is found",
-            False,
-            True,
-            "PDM_AUTO_GLOBAL",
+            env_var="PDM_CHECK_UPDATE",
             coerce=ensure_boolean,
         ),
         "build_isolation": ConfigItem(
@@ -100,6 +121,28 @@ class Config(MutableMapping[str, str]):
             "PDM_BUILD_ISOLATION",
             ensure_boolean,
         ),
+        "global_project.fallback": ConfigItem(
+            "Use the global project implicitly if no local project is found",
+            False,
+            True,
+            coerce=ensure_boolean,
+            replace="auto_global",
+        ),
+        "global_project.fallback_verbose": ConfigItem(
+            "If True show message when global project is used implicitly",
+            True,
+            True,
+            coerce=ensure_boolean,
+            replace=None,
+        ),
+        "global_project.path": ConfigItem(
+            "The path to the global project",
+            platformdirs.user_config_path("pdm") / "global-project",
+            True,
+        ),
+        "global_project.user_site": ConfigItem(
+            "Whether to install to user site", False, True, coerce=ensure_boolean
+        ),
         "project_max_depth": ConfigItem(
             "The max depth to search for a project through the parents",
             5,
@@ -116,7 +159,7 @@ class Config(MutableMapping[str, str]):
         "strategy.resolve_max_rounds": ConfigItem(
             "Specify the max rounds of resolution process",
             10000,
-            env_var="PDM_RESOLVE_MAX_ROUDNS",
+            env_var="PDM_RESOLVE_MAX_ROUNDS",
             coerce=int,
         ),
         "install.parallel": ConfigItem(
@@ -143,18 +186,22 @@ class Config(MutableMapping[str, str]):
         ),
         "python.use_venv": ConfigItem(
             "Install packages into the activated venv site packages instead of PEP 582",
-            False,
+            True,
             env_var="PDM_USE_VENV",
             coerce=ensure_boolean,
             replace="use_venv",
         ),
         "pypi.url": ConfigItem(
             "The URL of PyPI mirror, defaults to https://pypi.org/simple",
-            pypi_url,
+            DEFAULT_PYPI_INDEX,
             env_var="PDM_PYPI_URL",
         ),
         "pypi.verify_ssl": ConfigItem(
-            "Verify SSL certificate when query PyPI", verify_ssl, coerce=ensure_boolean
+            "Verify SSL certificate when query PyPI", True, coerce=ensure_boolean
+        ),
+        "pypi.ca_certs": ConfigItem(
+            "Path to a CA certificate bundle used for verifying the identity "
+            "of the PyPI server",
         ),
         "pypi.json_api": ConfigItem(
             "Consult PyPI's JSON API for package metadata",
@@ -162,12 +209,33 @@ class Config(MutableMapping[str, str]):
             env_var="PDM_PYPI_JSON_API",
             coerce=ensure_boolean,
         ),
+        "venv.location": ConfigItem(
+            "Parent directory for virtualenvs",
+            os.path.join(platformdirs.user_data_dir("pdm"), "venvs"),
+            global_only=True,
+        ),
+        "venv.backend": ConfigItem(
+            "Default backend to create virtualenv",
+            default="virtualenv",
+            env_var="PDM_VENV_BACKEND",
+        ),
+        "venv.in_project": ConfigItem(
+            "Create virtualenv in `.venv` under project root",
+            default=True,
+            env_var="PDM_VENV_IN_PROJECT",
+            coerce=ensure_boolean,
+        ),
     }
-    del pypi_url, verify_ssl
+
+    site: Config | None = None
 
     @classmethod
-    def get_defaults(cls) -> Dict[str, Any]:
-        return {k: v.default for k, v in cls._config_map.items() if v.should_show()}
+    def get_defaults(cls) -> dict[str, Any]:
+        defaults = {k: v.default for k, v in cls._config_map.items() if v.should_show()}
+        if cls.site is None:
+            cls.site = Config(platformdirs.site_config_path("pdm") / "config.toml")
+        defaults.update(cls.site)
+        return defaults
 
     @classmethod
     def add_config(cls, name: str, item: ConfigItem) -> None:
@@ -175,22 +243,24 @@ class Config(MutableMapping[str, str]):
         cls._config_map[name] = item
 
     def __init__(self, config_file: Path, is_global: bool = False):
-        self._data = {}
-        if is_global:
-            self._data.update(self.get_defaults())
-
         self.is_global = is_global
-        self._config_file = config_file.resolve()
-        self._file_data = load_config(self._config_file)
+        self.config_file = config_file.resolve()
         self.deprecated = {
             v.replace: k for k, v in self._config_map.items() if v.replace
         }
-        self._data.update(self._file_data)
+        self._file_data = load_config(self.config_file)
+        self._data = collections.ChainMap(
+            self._file_data, self.get_defaults() if is_global else {}
+        )
+
+    @property
+    def self_data(self) -> dict[str, Any]:
+        return dict(self._file_data)
 
     def _save_config(self) -> None:
         """Save the changed to config file."""
-        self._config_file.parent.mkdir(parents=True, exist_ok=True)
-        toml_data: Dict[str, Any] = {}
+        self.config_file.parent.mkdir(parents=True, exist_ok=True)
+        toml_data: dict[str, Any] = {}
         for key, value in self._file_data.items():
             *parts, last = key.split(".")
             temp = toml_data
@@ -200,10 +270,23 @@ class Config(MutableMapping[str, str]):
                 temp = temp[part]
             temp[last] = value
 
-        with self._config_file.open("w", encoding="utf-8") as fp:
+        with self.config_file.open("w", encoding="utf-8") as fp:
             tomlkit.dump(toml_data, fp)  # type: ignore
 
     def __getitem__(self, key: str) -> Any:
+        parts = key.split(".")
+        if parts[0] == REPOSITORY:
+            if len(parts) < 2:
+                raise PdmUsageError("Must specify a repository name")
+            repo = self.get_repository_config(parts[1])
+            if repo is None:
+                raise KeyError(f"No repository named {parts[1]}")
+
+            value = getattr(repo, parts[2]) if len(parts) >= 3 else repo
+            if len(parts) >= 3 and parts[2] == "password" and value:
+                return "<hidden>"
+            return value
+
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
         config_key = self.deprecated.get(key, key)
@@ -221,6 +304,17 @@ class Config(MutableMapping[str, str]):
         return config.coerce(result)
 
     def __setitem__(self, key: str, value: Any) -> None:
+        parts = key.split(".")
+        if parts[0] == REPOSITORY:
+            if len(parts) < 3:
+                raise PdmUsageError(
+                    "Set repository config with [green]repository.{name}.{attr}"
+                )
+            self._file_data.setdefault(parts[0], {}).setdefault(
+                parts[1], {}
+            ).setdefault(parts[2], value)
+            self._save_config()
+            return
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
         config_key = self.deprecated.get(key, key)
@@ -233,16 +327,13 @@ class Config(MutableMapping[str, str]):
         value = config.coerce(value)
         env_var = config.env_var
         if env_var is not None and env_var in os.environ:
-            click.echo(
-                termui.yellow(
-                    "WARNING: the config is shadowed by env var '{}', "
-                    "the value set won't take effect.".format(env_var)
-                )
+            ui.echo(
+                "WARNING: the config is shadowed by env var '{}', "
+                "the value set won't take effect.".format(env_var),
+                style="yellow",
             )
-        self._data[config_key] = value
         self._file_data[config_key] = value
         if config.replace:
-            self._data.pop(config.replace, None)
             self._file_data.pop(config.replace, None)
         self._save_config()
 
@@ -250,7 +341,7 @@ class Config(MutableMapping[str, str]):
         return len(self._data)
 
     def __iter__(self) -> Iterator[str]:
-        keys: Set[str] = set()
+        keys: set[str] = set()
         for key in self._data:
             if key in self._config_map:
                 keys.add(key)
@@ -259,22 +350,53 @@ class Config(MutableMapping[str, str]):
         return iter(keys)
 
     def __delitem__(self, key: str) -> None:
+        parts = key.split(".")
+        if parts[0] == REPOSITORY:
+            if len(parts) < 2:
+                raise PdmUsageError("Should specify the name of repository")
+            if len(parts) >= 3:
+                del self._file_data.get(REPOSITORY, {}).get(parts[1], {})[parts[2]]
+            else:
+                del self._file_data.get(REPOSITORY, {})[parts[1]]
+            self._save_config()
+            return
         config_key = self.deprecated.get(key, key)
         config = self._config_map[config_key]
-        self._data.pop(config_key, None)
         self._file_data.pop(config_key, None)
-        if self.is_global and config.should_show():
-            self._data[config_key] = config.default
         if config.replace:
-            self._data.pop(config.replace, None)
             self._file_data.pop(config.replace, None)
 
         env_var = config.env_var
         if env_var is not None and env_var in os.environ:
-            click.echo(
-                termui.yellow(
-                    "WARNING: the config is shadowed by env var '{}', "
-                    "set value won't take effect.".format(env_var)
-                )
+            ui.echo(
+                "WARNING: the config is shadowed by env var '{}', "
+                "set value won't take effect.".format(env_var),
+                style="yellow",
             )
         self._save_config()
+
+    def get_repository_config(self, name_or_url: str) -> RepositoryConfig | None:
+        """Get a repository by name or url."""
+        if not self.is_global:  # pragma: no cover
+            raise PdmUsageError("No repository config in project config.")
+        repositories: Mapping[str, Mapping[str, str | None]] = self._data.get(
+            REPOSITORY, {}
+        )
+        repo: RepositoryConfig | None = None
+        if "://" in name_or_url:
+            config: Mapping[str, str | None] = next(
+                (v for v in repositories.values() if v.get("url") == name_or_url), {}
+            )
+            repo = next(
+                (r for r in DEFAULT_REPOSITORIES.values() if r.url == name_or_url),
+                RepositoryConfig(name_or_url),
+            )
+        else:
+            config = repositories.get(name_or_url, {})
+            if name_or_url in DEFAULT_REPOSITORIES:
+                repo = DEFAULT_REPOSITORIES[name_or_url]
+        if repo:
+            return dataclasses.replace(repo, **config)
+        if not config:
+            return None
+        return RepositoryConfig(**config)  # type: ignore
